import logging
from .argumentparser import parse_args
from .downloader import download_playlist

# 48273

def main():
    '''Main driver function.
    '''
    args, additional_args = parse_args()
    download_playlist(args.playlist, args.directory, executable=args.executable,
                      executable_args=additional_args)
    logging.basicConfig(level=logging.INFO)

if __name__ == "__main__":
    main()



import subprocess
import logging
import os
from pathlib import Path
from typing import Dict, List

def kwargs_to_args(kwargs: Dict[str, str]) -> List[str]:
    '''Generate a list of arguments from keyword arguments.

    :param kwargs: Keyword arguments
    :type kwargs: Dict[str, str]
    :return: Arguments in order
    :rtype: List[str]
    '''
    args: list = []
    for key, arg in kwargs.items():
        if len(key) == 1:
            keyword = "-" + key
        else:
            keyword = "--" + key
        args.append(keyword)
        args.append(arg)
    return args

def download_playlist(playlist: str, output_directory: Path, executable: Path,
                      executable_args: list):
    '''Download playlist contents into a directory.

    :param playlist: Playlist URL.
    :type playlist: str
    :param output_directory: Output directory.
    :type output_directory: Path
    :param executable: Path to yt-dlp executable.
    :type executable: Path
    :param executable_args: Arguments to pass to executable
    :type executable_args: list
    '''
    # First, try to remove any options that could interfere with our script
    # from the executable args
    remove_slices: list = []
    invalid_options: list = ['-P', "--paths", '-a', '--batch-file', '-O', '--print',
                             '--newline', '--no-progress', '--progress', '--print-traffic',
                             '-v', '--verbose']
    for index, value in enumerate(executable_args):
        if(value in invalid_options):
            remove_slices.append((index, index+1))
    for remove_slice in reversed(remove_slices):
        del executable_args[remove_slice[0]:remove_slice[1]]
    
    # Then, append our own arguments
    additional_args: list = ['-P', str(output_directory), '--no-progress', '--print',
                             'after_move:filepath', '--', playlist]


    if not executable_args:
        executable_args = []
    for arg in additional_args:
        executable_args.append(arg)

    executable_args.insert(0, str(executable))
    
    logging.debug("Subprocess arguments: %s", executable_args)

    # Run the process and capture all filepaths
    filepaths: list = None
    with subprocess.Popen(executable_args,
                          stdout=subprocess.PIPE,
                          stderr=subprocess.PIPE,
                          stdin=subprocess.PIPE,
                          text=True) as process:
        output, error = process.communicate()
        return_code: int = process.returncode
        logging.debug("yt-dlp return code: %s", return_code)
        if return_code != 0:
            logging.critical("There was error running yt-dlp. See below:")
            logging.error("yt-dlp error: %s", error)
            return False
        output_lines: list = output.splitlines()
        filepaths = [Path(filepath.strip()).resolve() for filepath in output_lines]

    logging.debug("Extracted video paths: %s", filepaths)

    for dirpath, _, filenames in os.walk(output_directory):
        for filename in filenames:
            filepath: Path = Path(dirpath) / Path(filename)
            resolved: Path = filepath.resolve()
            logging.debug("Walked over: %s", resolved)
            if not (resolved in filepaths):
                logging.debug("Found extra file in directory: %s", filepath)
                logging.info("Removing file: %s", filepath)
                os.remove(resolved)

    logging.info("Syncing complete.")
    return True



import os
import logging
from shutil import which
from argparse import ArgumentParser, Namespace
from typing import Tuple
from urllib.parse import urlparse, urlunparse
from pathlib import Path

def url_type(potential_url: str) -> str:
    '''Custom URL argparse type that will do URL parsing to validate
    a URL argument.

    :param potential_url: Potential URL.
    :type potential_url: str
    :raises ValueError: Error raised if URL is malformed.
    :return: The URL if valid.
    :rtype: str
    '''
    parsed_url = urlparse(potential_url)
    if not any((parsed_url.scheme, parsed_url.netloc)):
        raise ValueError(f"URL \"{potential_url}\" is malformed.")
    return urlunparse(parsed_url)

def directory_type(potential_dir: Path) -> Path:
    '''Custom directory argparse type.

    :param potential_dir: Potential directory_
    :type potential_dir: Path
    :raises ValueError: Error raised if path is not a directory.
    :return: The resolved potential directory.
    :rtype: Path
    '''

    if not isinstance(potential_dir, Path):
        potential_dir = Path(potential_dir)
    # Create path if it doesn't exist
    if not potential_dir.exists():
        potential_dir.mkdir(parents=True, exist_ok=True)
    if not potential_dir.is_dir():
        raise ValueError(f"Path {potential_dir} is not a directory.")
    return potential_dir.resolve()

def executable_type(executable_path: Path) -> Path:
    '''Custom executable argparse type.

    :param executable_path: The executable path.
    :type executable_path: Path
    :raises ValueError: Error raised if path is not an existing executable file.
    :return: The path resolved.
    :rtype: Path
    '''
    if not isinstance(executable_path, Path):
        executable_path = Path(executable_path)
    if not executable_path.exists():
        raise ValueError(f"Path {executable_path} does not exist.")
    if not executable_path.is_file():
        raise ValueError(f"Path {executable_path} is not a file.")
    if not os.access(executable_path, os.X_OK):
        raise ValueError(f"Path {executable_path} is not executable.")
    return executable_path.resolve()

def parse_args() -> (Namespace, Namespace):
    '''Create an argument parser and parse args.

    :return: The parsed arguments, and unparsed arguments.
    :rtype: Namespace
    '''
    # Create argument parser
    argparser: ArgumentParser = ArgumentParser(
                                    prog="yt-p-sync",
                                    description="Sync a YouTube playlist.",
                                    epilog="Any additional options will be "
                                           "passed directly to your yt-dlp "
                                           "executable."
                                )
    default_executable: str = which("yt-dlp")
    argparser.add_argument("--executable", type=executable_type, required=False,
                           help="Path to yt-dlp executable.", default=default_executable)
    argparser.add_argument("--logging", help="Logging level. Default 20 (INFO).", default=logging.INFO,
                           choices=(logging.DEBUG, logging.INFO, logging.WARNING, logging.ERROR, logging.CRITICAL))
    argparser.add_argument("playlist", type=url_type, help="URL of playlist to sync.")
    argparser.add_argument("directory", type=directory_type, help="Directory to sync playlist to.")

    # Parse arguments and return results
    args, unknown_args = argparser.parse_known_args()

    return (args, unknown_args)
